# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::Comment;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::DB;


=item add_unsafe
Add a comment.
Does not use transactions.

0: text
1: entry id
2: parent comment id
(3: nick)
(4: user ip)
(5: user browser user agent)
=cut
sub add_unsafe($$$$$$) {
	my ($text, $entry, $parent, $nick, $net_ip, $net_agent) = @_;

	defined($text) || die "Comment: adding without text!";
	defined($entry) || die "Comment: adding without entry id!";

	$parent = -1		unless defined($parent);
	$nick = "Anonymous"	unless defined($nick) && $nick ne "";

	($text eq "") && return;

	# See if /published/ entry exists
	my $entry_public = &Woclema::Entry::exists($entry, 1);
	defined($entry_public) || die "Comment::add: No such entry: $entry";
	($entry_public == 0) && die "Comment::add: " .
			"Commenting unpublished entry $entry";
	
	# See if parent exists
	if ($parent != -1) {
		my %info = &get_info($parent);
		defined($info{'id'}) || die "Comment::add: " .
				"Parent comment $parent doesn't exist";
	}
	
	# Oh the horror!
	my $columns = "text, entry, parent, nick";
	my $values = "&sa&, &db&, &dc&, &sd&";

	if (defined($net_ip)) {
		$columns .= ", net_ip";
		$values .= ", &se&";
	}
	if (defined($net_agent)) {
		$columns .= ", net_agent";
		$values .= ", &sf&";
	}

	my $update = &Woclema::DB::prepare(
			"INSERT INTO comment ($columns) VALUES ($values)");
	$update = &Woclema::DB::set($update,
			"a", $text,
			"b", $entry,
			"c", $parent,
			"d", $nick);
	
	$update = &Woclema::DB::set($update, "e", $net_ip)
			if (defined($net_ip));
	$update = &Woclema::DB::set($update, "f", $net_agent)
			if (defined($net_agent));

	&Woclema::DB::update($update);

	# Increment comment-counter for the entry
	&Woclema::Entry::comment_counter_unsafe($entry, 1);
}


=item add
Add a comment.

0: text
1: entry id
2: parent comment id
(3: nick)
(4: user ip)
(5: user browser user agent)
=cut
sub add($$$$$$) {
	&Woclema::DB::transaction_start();
	&add_unsafe(@_);
	&Woclema::DB::transaction_end();
}


=item get_info
Get comment info.

0: id
r: array of comments
=cut
sub get_for_entry($) {
	my ($id) = @_;
	my ($query, $result);

	$query = &Woclema::DB::prepare(
			"SELECT text, nick, parent, entry, " .
			"net_ip, net_agent, " .
		"EXTRACT(YEAR FROM timest AT TIME ZONE 'UTC') AS year, " .
		"EXTRACT(MONTH FROM timest AT TIME ZONE 'UTC') AS month, " .
		"EXTRACT(DAY FROM timest AT TIME ZONE 'UTC') AS day, " .
		"EXTRACT(HOUR FROM timest AT TIME ZONE 'UTC') AS hour, " .
		"EXTRACT(MINUTE FROM timest AT TIME ZONE 'UTC') AS minute, " .
		"EXTRACT(SECOND FROM timest AT TIME ZONE 'UTC') AS second, " .
		"EXTRACT(EPOCH FROM timest) AS epoch " .
			"FROM comment WHERE entry = &da& ORDER BY id");
	$query = &Woclema::DB::set($query, "a", $id);
	$result = &Woclema::DB::query($query);

	my @ret;
	while (my ($text, $nick, $parent, $entry, $net_ip, $net_agent,
					$year, $month, $day,
					$hour, $minute, $second, $epoch) =
			&Woclema::DB::get_row($result)) {
		my %line = (
			"id"		=> $id,
			"text"		=> $text,
			"nick"		=> $nick,
			"parent"	=> $parent,
			"entry"		=> $entry,
			"net_ip"	=> $net_ip,
			"net_agent"	=> $net_agent,
			"time_year"	=> $year,
			"time_month"	=> $month,
			"time_day"	=> $day,
			"time_hour"	=> $hour,
			"time_minute"	=> $minute,
			"time_second"	=> $second,
			"time_epoch"	=> $epoch
			);
		push(@ret, \%line);
	}

	return @ret;
}


return 1;
