#! /usr/bin/perl
#
# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::Entry;
use Woclema::Category;
use Woclema::Comment;

use Woclema::Summary;
use Woclema::Calendar;
use Woclema::Link;

use Woclema::HTML;
use Woclema::HTTP;

# URL-format:
# weblog-pl?...
#
# Single entry (show entry with comments):
#   entry	entry ID
#
# Send comment (show entry with comments):
#   entry	entry to comment
#   parent	parent to refer
#   data	comment content
#   nick	commentor nick
#   net_ip	commentor IP
#   net_agent	commentor web-browser user agent
#
# Show day (print entries without comments):
#   day		unix timestamp (epoch)
#
# Show list by ID (print entries without comments):
#   latest	entry ID
#   n
#
# Show month (brief list):
#   month	unix timestamp (epoch)
#
# Show entries in category:			# TODO
#   cat		category ID
#   latest	entry ID
#   n


my $dsec = 24 * 60 * 60;


=item post_comment
Post comment (get data from POST-data).
=cut
sub post_comment() {
	my $entry = &Woclema::HTTP::post("entry");
	my $parent = &Woclema::HTTP::post("parent");
	my $text = &Woclema::HTTP::post("data");
	my $nick = &Woclema::HTTP::post("nick");
	my $ip = &Woclema::HTTP::get_ip();
	my $agent = &Woclema::HTTP::get_agent();

	return unless (defined($entry) && defined($text));
	$text =~ s/^\s+//g;
	$text =~ s/\s+$//g;
	return if ($text eq "");

	foreach ($entry, $parent, $text, $nick, $agent) {
		next unless defined($_);
		s/</&lt;/g;
		s/>/&gt;/g;
		s/\t/ /g;
		s/ +/ /g;
		s/\r\n/\n/g;
		s/\n\r/\n/g;
		s/\r/\n/g;
	}

	foreach ($entry, $parent, $nick, $agent) {
		next unless defined($_);
		s/\n//g;
	}

	# Use <p> for comments
	$text =~ s/\n\n+/\r/g;
	$text =~ s/\n/ /g;
	my $t = join("</p><p>", split(/\r/, $text));
	$text = "<p>$t</p>";

	&Woclema::Comment::add($text, $entry, $parent, $nick, $ip, $agent);
}


=item print_calendar
Print calendar.

0: given day, default to time()
1: first day of week
r: calendar HTML with links
=cut
sub print_calendar($) {
	my ($given, $fdow) = @_;
	my ($first, $last);

	$given = time() unless defined($given);

	($first, $last) =
		&Woclema::Calendar::get_cont_month_limits($given, $fdow);
	my @given_date = gmtime($given);

	my $out = "<div class=\"calendar\">\n";
	$out .= "<table>\n";
	$out .= sprintf("<caption><a href=\"%s\">%s</a> %d</caption>",
			&Woclema::Link::month($given),
			&Woclema::Calendar::month_name($given_date[4] + 1),
			$given_date[5] + 1900);
	$out .= "\t<tr>\n";
	for (my $i = 1; $i < 8; $i++) {
		$out .= sprintf("\t\t<th>%s</th>\n",
				&Woclema::Calendar::day_short($i % 7));
	}
	$out .= "\t</tr>\n";

	my @entries = &Woclema::Entry::exist_range($first, $last);

	my $this = $first;
	for (my $y = $first; $y < $last; $y += 7 * $dsec) {
		$out .= "\t<tr>\n";
		for (my $x = 0; $x < 7; $x++) {
			my @date = gmtime($this);
			# class. may have the following "flags" in this order:
			#   m: not this month
			#   t: today
			#   e: has entry
			my $class;
			$class .= "m" if ($date[4] != $given_date[4]);

			my $link;
			my $this_end = $this + $dsec;
			$class .= "t" if (time() >= $this
					&& time() < $this_end);
			foreach my $entry (@entries) {
				if ($entry >= $this && $entry < $this_end) {
					$class .= "e";
					$link = $this;
					last;
				}
			}

			$class = defined($class) ? " class=\"$class\"" : "";
			if (defined($link)) {
				$out .= sprintf("\t\t<td%s>" .
						"<a href=\"%s\">%s</a>" .
						"</td>\n",
						$class,
						&Woclema::Link::day($link),
						$date[3]);
			} else {
				$out .= sprintf("\t\t<td%s>%d</td>\n",
						$class,
						$date[3]);
			}
			$this += $dsec;
		}
		$out .= "\t</tr>\n";
	}

	$out .= "</table>\n";
	$out .= "</div>\n";

	return $out;
}


=item get_cat_links
Get HTML for showing entry categories.

0: entry ID
r: HTML
=cut
sub get_cat_links($) {
	my ($id) = @_;

	my @cats_list = &Woclema::Category::get_for_entry($id);
	my $cats = "";
	while ($#cats_list >= 1) {
		my $cat_id = shift @cats_list;
		my $cat_name = shift @cats_list;
		$cats .= sprintf(", <a href=\"%s\">%s</a>",
				&Woclema::Link::category($cat_id),
				$cat_name);
	}
	$cats =~ s/^, //;
	return $cats;
}


=item get_gen_time
Get timestamp for entry/comment.

0: entry/comment
1: brief if defined and set 1
r: time
=cut
sub get_gen_time(%) {
	my ($raw, $brief) = @_;
	my %e = %{($raw)};
	if (defined($brief) && $brief == 1) {
		return sprintf("%d %s %d",
				$e{'time_day'},
				&Woclema::Calendar::month_name(
					$e{'time_month'}),
				$e{'time_year'});
	} else {
		return sprintf("%d %s %d %02d:%02d:%02d",
				$e{'time_day'},
				&Woclema::Calendar::month_name(
					$e{'time_month'}),
				$e{'time_year'},
				$e{'time_hour'},
				$e{'time_minute'},
				$e{'time_second'});
	}
}


=item add_sidebar_cats
Add category-links to sidebar.
=cut
sub add_sidebar_cats() {
	my @cats = &Woclema::Summary::entries_per_category();
	return if ($#cats < 0);

	my $out = "<div class=\"cats\">\n";
	$out .= "Categories";
	$out .= "<ul>\n";

	my @alpha;
	foreach (@cats) {
		my @t = split(/\t/, $_, 3);
		push(@alpha, uc($t[2]));
	}
	my @cats_s = @cats[ sort { $alpha[$a] cmp $alpha[$b] } 0..$#cats ];

	foreach (@cats_s) {
		my ($id, $count, $name) = split(/\t/, $_, 3);
		$out .= sprintf("\t<li><a href=\"%s\">%s (%d)</a></li>\n",
				&Woclema::Link::category($id),
				&Woclema::HTML::htmlize($name),
				$count);
	}
	$out .= "</ul>\n";
	$out .= "</div>\n";

	&Woclema::HTML::add_sidebar($out);
}


=item add_sidebar_months
Add link to recent months in sidebar.
=cut
sub add_sidebar_months() {
	my @entries = &Woclema::Summary::all_time_entries(
			$Woclema::Config::old_months);

	($#entries < 0) && return;

	my $out = "<div class=\"archive\">\n";
	$out .= "Archive\n";
	$out .= "<ul>\n";

	for (my $i = 0; $i <= $#entries; $i++) {
		my ($count, $year, $month, $epoch) = split(/\t/, $entries[$i]);
		$out .= sprintf("\t<li><a href=\"%s\">%s %d (%d)</a></li>\n",
				&Woclema::Link::month($epoch),
				&Woclema::Calendar::month_name($month),
				$year,
				$count);
	}
	$out .= sprintf("\t<li class=\"all\">" .
			"<a href=\"%s\">All entries</a></li>\n",
			&Woclema::Link::summary());
	$out .= "</ul>\n";
	$out .= "</div>\n";

	&Woclema::HTML::add_sidebar($out);
}


=item print_comment_rec
Print comment.

0: "this" comment
1: parent comment ID
...: all comments
=cut
sub print_comment_rec($$@) {
	my $comm_raw = shift;
	my $parent = shift;
	my @list = @_;

	my %comm = %{($comm_raw)};
	return unless ($comm{'parent'} == $parent);

	my $time = &get_gen_time(\%comm);
	&Woclema::HTML::add_content(sprintf "%s", <<EOF);
<div class="comment">
<span class="time">$time</span>: 
<span class="nick">$comm{'nick'}</span>
<div class="text">
$comm{'text'}
</div>

EOF

	foreach (@list) {
		&print_comment_rec($_, $comm{'id'}, @list);
	}

	&Woclema::HTML::add_content("</div>");
}


=item print_comments
Print weblog entry comments and a box to enter a new comment.

0: entry ID
=cut
sub print_comments($) {
	my ($id) = @_;

	# Print entry box
	my $url = &Woclema::Link::entry($id);
	&Woclema::HTML::add_content(sprintf "%s", <<EOF);
<div id="submit">
<h3>Post comment</h3>
<form action="$url" method="post">
<p>
Nick: <input name="nick" size="40" /><br />
<textarea name="data" cols="80" rows="8">
</textarea><br />
<input type="submit" value="Post comment" />
<input type="hidden" name="entry" value="$id" />
</p>
</form>
</div>

EOF

	my @comm = &Woclema::Comment::get_for_entry($id);
	return if ($#comm == -1);

	&Woclema::HTML::add_content("<h3>Comments</h3>");
	&print_comment_rec($_, -1, @comm) foreach (@comm);
}


=item print_entry
Print single entry.

0: entry
1: show comments
=cut
sub print_entry(\%$) {
	my ($entry_raw, $comments) = @_;

	my %entry = %{($entry_raw)};
	my $cats = &get_cat_links($entry{'id'});

	my $links = sprintf("<a href=\"%s\">Permalink</a>",
			&Woclema::Link::entry($entry{'id'}));
	if (! defined($comments) || $comments == 0) {
		$links .= sprintf(" | <a href=\"%s\">Comments</a> (%d)",
				&Woclema::Link::entry($entry{'id'}),
				$entry{'comments'});
	}

	my $time = &get_gen_time(\%entry);
	&Woclema::HTML::add_content(sprintf "%s", <<EOF);
<div class="entry">
<h2>$entry{'title'}</h2>
<span class="date">$time</span>
<span class="cats">$cats</span>
$entry{'text'}
<span class="links">$links</span>
EOF

	&print_comments($entry{'id'}) if (defined($comments) && $comments == 1);

	&Woclema::HTML::add_content("</div>\n");
}


=item print_list_summary
Print summary of given entries.

...: entries
=cut
sub print_list_summary(@) {
	my @list = @_;
	my $tdate;

	my $out = "<div class=\"list\">\n";
	$out .= "<ul>\n";

	foreach (@list) {
		my %entry = %{($_)};
		my $date = &get_gen_time(\%entry, 1);
		my $cats = &get_cat_links($entry{'id'});

		if (defined($tdate) && $tdate ne $date) {
			$out .= "</ul></li>\n";
		}
			
		if (! defined($tdate) || $tdate ne $date) {
			$out .= sprintf("<li><span class=\"date\">" .
					"<a href=\"%s\">%s</a></span>" .
        				"<ul>\n",
				&Woclema::Link::day($entry{'time_epoch'}),
				$date);
			$tdate = $date;
		}

		my $link = &Woclema::Link::entry($entry{'id'});
		my $time = sprintf("%02d:%02d:%02d",
				$entry{'time_hour'},
				$entry{'time_minute'},
				$entry{'time_second'});
		$out .= sprintf("%s", <<EOF);
	<li>
		<span class="time">$time</span>
		<span class="title"><a href="$link">$entry{'title'}</a></span>
		<span class="cats">(in $cats)</span>
	</li>
EOF
	}

	$out .= "</li></ul>\n";
	$out .= "</div>";
	&Woclema::HTML::add_content($out);
}


=item print_list
Print entries.

0: verbose limit
1: automagic for single entry
...: entries
r: date of the latest entry
=cut
sub print_list($$@) {
	my $limit = shift;
	my $automagic = shift;
	my @list = @_;

	($#list < 0) && return time() ;

	my %first = %{($list[0])}; # print_entry screws up entries...

	if ($#list == 0 && $automagic == 1) {
		&print_entry($list[0], 1);
	} elsif ($#list < $limit) {
		&print_entry($_, 0) foreach (@list);
	} else {
		&print_list_summary(@list);
	}

	return $first{'time_epoch'};
}


=item print_list
Print list of entries.

0: verbose limit
1: automagic for single entry
2: start date
3: end date
r: date of the latest entry
=cut
sub print_time_range($$$) {
	my ($limit, $automagic, $first, $last) = @_;
	my @list = &Woclema::Entry::get_list_between_time($first, $last);
	my $date = &print_list($limit, $automagic, @list);
	return $date;
}


# URL-parameter handlers


=item show_entry
Show single log entry (with comments and comment entry box).
Also used to parse sending comments.

0: entry ID
r: date of entry
=cut
sub show_entry($) {
	my ($id) = @_;

	my %entry = &Woclema::Entry::get($id);
	return unless defined($entry{'id'});

	&print_entry(\%entry, 1);

	return $entry{'time_epoch'};
}


=item show_latest
Show list of N entries without comments starting from ID N.

0: latest entry timestamp (epoch), defaults to time()
1: number of entries, defaults to Config::list_verbose_limit
r: date of the latest entry
=cut
sub show_latest($$) {
	my ($latest, $n) = @_;
	$latest = time() unless defined($latest);
	$n = $Woclema::Config::latest_count unless defined($n);

	my @list = &Woclema::Entry::get_list_before_time($latest, $n);
	my $date = &print_list($n, $Woclema::Config::verbose_single_latest,
			@list);
	return $date;
}


=item show_day
Show entries of given day.

0: date (UNIX epoch)
r: date of the latest entry
=cut
sub show_day($) {
	my ($day) = @_;
	my ($first, $last) = &Woclema::Calendar::get_day_limits($day);
	my $date = &print_time_range($Woclema::Config::verbose_limit_day,
			$Woclema::Config::verbose_single_day,
			$first, $last);
	return $date;
}


=item show_month
Show list of entries of given month.

0: month
r: date of the latest entry
=cut
sub show_month($) {
	my ($month) = @_;
	my ($first, $last) = &Woclema::Calendar::get_month_limits($month);
	my $date = &print_time_range($Woclema::Config::verbose_limit_month,
			$Woclema::Config::verbose_single_month,
			$first, $last);
	return $date;
}


=item show_cat_entries
Show list of entries in given category.

0: category ID
=cut
sub show_cat_entries($) {
	my ($cat) = @_;
	my $name = &Woclema::Category::get_name($cat);
	if (defined($name)) {
		&Woclema::HTML::add_content(
			"<h2>Entries in &quot;$name&quot;</h2>");
	} else {
		&show_latest();
	}

	my @list = &Woclema::Entry::get_for_cat($cat);
	&Woclema::HTML::add_content();
	&print_list($Woclema::Config::verbose_limit_category,
			$Woclema::Config::verbose_single_category,
			@list);
}


=item show_summary
Show brief list of all entries per month/year.
=cut
sub show_summary() {
	my @list = &Woclema::Summary::all_time_entries(4096); # default: 12
	my $list_i = 0;

	($#list == -1) && return;

	my $out = "<div class=\"summary\">\n";
	$out .= "<table class=\"summary\">\n";

	my @first = gmtime();
	my $first_year = $first[5] + 1900;
	my $first_month = $first[4] + 1;
	my @last = split(/\t/, $list[$#list]);
	my $last_year = $last[1];
	my $last_month = $last[2];

	# In case there are entries in future, we need to adjust @first
	my @f_entry = split(/\t/, $list[0]);
	if ($f_entry[1] > $first_year) {
		$first_year = $f_entry[1];
		$first_month = $f_entry[2];
	} elsif ($f_entry[1] == $first_year && $f_entry[2] > $first_month) {
		$first_month = $f_entry[2];
	}

	for (my $year = $first_year; $year >= $last_year; $year--) {
		$out .= "\t<tr><th colspan=\"2\">$year</th></tr>\n";

		my $mon_s = $year != $first_year ? 12 : $first_month;
		my $mon_e = $year != $last_year ? 1 : $last_month;
	
		for (my $month = $mon_s; $month >= $mon_e; $month--) {
			my ($e_count, $e_year, $e_month, $e_epoch) =
				split(/\t/, $list[$list_i]);
			my ($c0, $c1, $class);
			if ($e_year == $year && $e_month == $month) {
				# "count" == 0 is impossible!
				my ($t0, $t1);
				$t0 = &Woclema::Link::month($e_epoch);
				$t1 = &Woclema::Calendar::month_name($month);
				$c0 = sprintf("<a href=\"%s\">%s</a>",
						$t0, $t1);
				$c1 = &Woclema::Common::make_unit(
						$e_count, "entry", "entries");
				$list_i++;
				$class = "";
			} else {
				$c0 = &Woclema::Calendar::month_name($month);
				$c1 = "no entries";
				$class = " class=\"no\"";
			}
			$out .= "\t<tr$class>\n";
			$out .= "\t\t<td>$c0</td>\n";
			$out .= "\t\t<td>$c1</td></tr>\n";
		}
	}
	$out .= "</table>\n";
	$out .= "</div>\n";

	&Woclema::HTML::add_content($out);
}


=item main
Main function.
=cut
sub main() {
	my $entry = &Woclema::HTTP::get_numeric("entry");
	my $day = &Woclema::HTTP::get_numeric("day");
	my $month = &Woclema::HTTP::get_numeric("month");
	my $latest = &Woclema::HTTP::get_numeric("n");
	my $summary = &Woclema::HTTP::get_numeric("summary");
	my $cat = &Woclema::HTTP::get_numeric("cat");

	&post_comment() if (defined(&Woclema::HTTP::post("data")));

	my $cal_date = time();

	if (defined($entry)) {
		$cal_date = &show_entry($entry);
	} elsif (defined($day)) {
		$cal_date = &show_day($day);
	} elsif (defined($month)) {
		$cal_date = &show_month($month);
	} elsif (defined($summary)) {
		&show_summary();
	} elsif (defined($cat)) {
		&show_cat_entries($cat);
	} elsif (defined($latest)) {
		my $n = &Woclema::HTTP::get_numeric("n");
		$cal_date = &show_latest($latest, $n);
	} else {
		$cal_date = &show_latest();
	}

	&Woclema::HTML::add_sidebar(&print_calendar($cal_date));
	&add_sidebar_cats();
	&add_sidebar_months();
#	&add_sidebar_extras();

	&Woclema::HTTP::print_header();
	&Woclema::HTML::print();
}


&main();
exit(0);
